const express = require('express');
const mysql = require('mysql');
const moment = require('moment');
const multer = require('multer');
const cookieParser = require('cookie-parser')
const path = require('path');
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        cb(null, '../../fromus_media');
    },
    filename: (req, file, cb) => {
        cb(null, Date.now() + file.originalname)
    }
})
const upload = multer({ 
    storage: storage,
    fileFilter: function (req, file, callback) {
        var ext = path.extname(file.originalname);
        if(ext !== '.png' && ext !== '.jpg' && ext !== '.pdf' && ext !== '.jpeg') {
            return callback(new Error('Only images are allowed'))
        }
        callback(null, true)
    },
});
const postsFunctions = require('../functions/posts');
const authjs = require('../functions/auth');
const commentsjs = require('../functions/comments');
const fs = require('fs');
var router = express.Router();
router.use(cookieParser());

router.get('/:postId', async (req, res) => {
    var cookies = req.cookies;
    var body = req.body;
    var userId = cookies.c_id;
    var userToken = cookies.c_t;
    var postId = req.params.postId;
    if (!postId) {
        res.json({ status: 400, message: 'Bad Request' });
        return;
    }

    var existence = await postsFunctions.checkPostExistence(postId);
    if (!existence) {
        res.json({ status: 404, message: 'Post does not exist' });
        return;
    }

    var query = `
    SELECT 
    comments.*,
    users.name, users.profile_picture, users.id as user_id,
    replies.replies as replies_count,
    awareness_points.count as awareness_points
    FROM posts_comments comments
    LEFT JOIN users users
    ON comments.user_id = users.id
    LEFT JOIN LATERAL 
    (
        SELECT COUNT(*) as replies, parent_id 
        FROM posts_comments
        WHERE parent_id = comments.id
        GROUP BY parent_id
    ) as replies
    ON comments.id = replies.parent_id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, comment_id
        FROM posts_comments_likes
        WHERE comment_id = comments.id
        GROUP BY comment_id
    ) as awareness_points
    ON comments.id = awareness_points.comment_id
    WHERE comments.post_id = ?
    ORDER BY date DESC
    LIMIT 25
    `;
    var query_params = [postId];
    if(userId != undefined && userToken != undefined){
        var auth = await authjs.checkToken(userId, userToken);
        if (auth == false) {
            res.json({ status: 401, message: 'Failed to authenticate' });
            return;
        }
        query = `SELECT 
        comments.*,
        users.name, users.profile_picture, users.id as user_id,
        replies.replies as replies_count,
        awareness.aware as aware,
        awareness_points.count as awareness_points
        FROM posts_comments comments
        LEFT JOIN users users
        ON comments.user_id = users.id
        LEFT JOIN LATERAL 
        (
            SELECT COUNT(*) as replies, parent_id 
            FROM posts_comments
            WHERE parent_id = comments.id
            GROUP BY parent_id
        ) as replies
        ON comments.id = replies.parent_id
        LEFT JOIN LATERAL 
        (
            SELECT COUNT(*) as aware, comment_id 
            FROM posts_comments_likes
            WHERE comment_id = comments.id
            AND user_id = ?
        ) as awareness
        ON comments.id = awareness.comment_id
        LEFT JOIN LATERAL
        (
            SELECT COUNT(*) as count, comment_id
            FROM posts_comments_likes
            WHERE comment_id = comments.id
            GROUP BY comment_id
        ) as awareness_points
        ON comments.id = awareness_points.comment_id
        WHERE comments.post_id = ?
        ORDER BY date DESC
        LIMIT 25`;
        query_params = [userId, postId];
    }
    pool.query(query, query_params, async (err, result) => {
        if (err) {
            res.json({ status: 500, message: 'Internal Server Error', err });
            return;
        }
        for (const comment of result) {

            if (comment.awareness_points == null) {
                comment.awareness_points = 0;
            }

            if (comment.aware == 0) {
                comment.aware = false;
            } else {
                comment.aware = true;
            }

            if (comment.replies_count == null) {
                comment.replies_count = 0;
            }

            if(comment.parent_id != null){
                var parentComment = await commentsjs.commentExistence(comment.id, false);
                if(parentComment != false){
                    comment.parent_comment = parentComment;
                }
            }

            comment.formatted_date = new Date(comment.date).toLocaleDateString('en-us', { year:"numeric", month:"long", day:"numeric"})
        }
        res.json({ status: 200, comments: result });
    });
});

router.post('/create', upload.none(), async (req, res) => {
    var cookies = req.cookies;
    var body = req.body;
    var userId = cookies.c_id;
    var userToken = cookies.c_t;
    var post_id = body.postId;
    var content = body.content;
    var parent_id = body.parentId;
    if (!userId || !userToken || !content) {
        res.json({ status: 400, message: 'Bad Request' });
        return;
    }
    if(content.lengh == 0 || content.trim() == 0)
    if(content.length > 500){
        res.json({status:400, error:'Content Long', message:"Your comment is too long"});
        return;
    }
    if (parent_id && post_id) {
        //IF PARENT_ID IS PROVIDED, POST_ID MUST NOT BE PROVIDED
        res.json({ status: 400, message: 'Bad Request' });
        return;
    } else if (!parent_id && !post_id) {
        //IF PARENT_ID IS NOT PROVIDED, POST_ID MUST BE PROVIDED
        res.json({ status: 400, message: 'Bad Request' });
        return;
    }

    var auth = await authjs.checkToken(userId, userToken);
    if (auth == false) {
        res.json({ status: 401, message: 'Failed to authenticate' });
        return;
    }

    if (post_id) {
        var existence = await postsFunctions.checkPostExistence(post_id);
        if (!existence) {
            res.json({ status: 404, message: 'Post does not exist' });
            return;
        }
    }
    if (parent_id) {
        var existence = await commentsjs.commentExistence(parent_id, false);
        if (existence == false) {
            res.json({ status: 404, message: 'Comment does not exist' });
            return;
        }
        post_id = existence.post_id;
    }

    var date = moment().format('YYYY-MM-DD HH:mm:ss');
    var comment_id = parseInt(new Date().valueOf().toString().substr(0,10));
    var comment = {
        id:comment_id,
        content,
        post_id,
        user_id:userId,
        parent_id,
        date
    };

    var query = 'INSERT INTO posts_comments SET ?';
    pool.query(query, comment, async (err, results) => {
        if(err){
            res.json({status:500, error:'Internal Server Error', message:'Sorry Something Went Wrong', err});
        }else{
            var getComment = await commentsjs.commentExistence(comment_id, false);
            if(getComment == false){
                res.json({status:200, message:'Ok', message:"Could not retrieve comment"});
                return;
            }
            comment = getComment;
            if (comment.awareness_points == null) {
                comment.awareness_points = 0;
            }

            if (comment.aware == null) {
                comment.aware = false;
            } else {
                comment.aware = true;
            }

            if (comment.replies_count == null) {
                comment.replies_count = 0;
            }

            if(comment.parent_id != null){
                var parentComment = await commentsjs.commentExistence(comment.id, false);
                if(parentComment != false){
                    comment.parent_comment = parentComment;
                }
            }

            comment.formatted_date = new Date(comment.date).toLocaleDateString('en-us', { year:"numeric", month:"long", day:"numeric"})

            res.json({status:200, message:'Ok', data:getComment});
        }
    });
});

router.post('/like', upload.none(), async (req, res) => {
    var cookies = req.cookies;
    var body = req.body;
    var id = cookies.c_id;
    var token = cookies.c_t;
    var comment_id = body.comment_id;

    if(!id || !token || !comment_id){
        res.json({ status: 400, message: 'Bad Request' });
        return;
    }

    var auth = await authjs.checkToken(id, token);
    if (auth == false) {
        res.json({ status: 401, message: 'Failed to authenticate' });
        return;
    }

    var existence = await commentsjs.commentExistence(comment_id);
    if(existence == false){
        res.json({ status: 400, message: 'Provide a valid comment id' });
        return;
    }

    var aware = await commentsjs.commentAwareness(comment_id, id);
    if(aware == true){
        res.json({status:200, message:"Ok"});
        return;
    }

    var query = 'INSERT INTO posts_comments_likes (user_id, comment_id, date) VALUES (?, ?, ?)';
    var date = moment().format('YYYY-MM-DD HH:mm:ss');
    pool.query(query, [id, comment_id, date], (err, results) => {
        if(err){
            res.json({status:500, error:'Internal Server Error', message:'Sorry Something Went Wrong'});
        }else{
            res.json({status:200, message:"Ok"});
        }
    });
});

router.post('/unlike', upload.none(), async (req, res) => {
    var cookies = req.cookies;
    var body = req.body;
    var id = cookies.c_id;
    var token = cookies.c_t;
    var comment_id = body.comment_id;

    if(!id || !token || !comment_id){
        res.json({ status: 400, message: 'Bad Request' });
        return;
    }

    var auth = await authjs.checkToken(id, token);
    if (auth == false) {
        res.json({ status: 401, message: 'Failed to authenticate' });
        return;
    }

    var existence = await commentsjs.commentExistence(comment_id);
    if(existence == false){
        res.json({ status: 400, message: 'Provide a valid comment id' });
        return;
    }

    var aware = await commentsjs.commentAwareness(comment_id, id);
    if(aware == false){
        res.json({status:200, message:"Ok"});
        return;
    }

    var query = 'DELETE FROM posts_comments_likes WHERE comment_id = ? AND user_id = ?';
    var date = moment().format('YYYY-MM-DD HH:mm:ss');
    pool.query(query, [comment_id, id], (err, results) => {
        if(err){
            res.json({status:500, error:'Internal Server Error', message:'Sorry Something Went Wrong'});
        }else{
            res.json({status:200, message:"Ok"});
        }
    });
});

module.exports = router;
//import mysql pool
const pool = require('../database');