const express = require('express');
const mysql = require('mysql');
const moment = require('moment');
const cookieParser = require('cookie-parser')
const multer = require('multer');
const path = require('path');
const storage = multer.diskStorage({
    destination: (req, file, cb) => {
        cb(null, '../../fromus_media');
    },
    filename: (req, file, cb) => {
        cb(null, Date.now() + file.originalname)
    }
})
const upload = multer({ 
    storage: storage,
    fileFilter: function (req, file, callback) {
        var ext = path.extname(file.originalname);
        if(ext !== '.png' && ext !== '.jpg' && ext !== '.pdf' && ext !== '.jpeg') {
            return callback(new Error('Only images are allowed'))
        }
        callback(null, true)
    },
});

var router = express.Router();
const postFunctions = require('../functions/posts');
const postModels = require('../models/post');
const authjs = require('../functions/auth');
router.use(cookieParser());

router.use('/comments', require('./comments'));

router.get('/feed', async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    var query = `SELECT 
    posts.id as post_id, posts.content, posts.date,
    users.name, users.profile_picture, users.uid as username, users.graduation_year,
    likes.count as likes,
    liked.liked as liked,
    comments_count.count as comments_count
    FROM posts posts
    LEFT JOIN users users
    ON posts.user_id = users.id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as liked, post_id
        FROM posts_likes
        WHERE post_id = posts.id
        AND user_id = ?
    )as liked
    ON posts.id = liked.post_id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, post_id
        FROM posts_likes
        WHERE post_id = posts.id
        GROUP BY post_id
    )as likes
    ON posts.id = likes.post_id
    LEFT JOIN LATERAL
    (
        SELECT COUNT(*) as count, post_id
        FROM posts_comments
        WHERE post_id = posts.id
        GROUP BY post_id
    ) as comments_count
    ON posts.id = comments_count.post_id
    WHERE posts.id IS NOT NULL
    ORDER BY date DESC
    LIMIT 15`;
    pool.query(query, [id], async (err, results) => {
        var modified_results = [];
        for (let i = 0; i < results.length; i++) {
            var post_element = results[i];
            if(post_element.post_id == null){
                continue;
            }
            post_element = new postModels.Post(post_element);
            var media = await postFunctions.getMedia(post_element.post_id);
            post_element.media = media;
            post_element.media_count = media.length;
            modified_results.push(post_element)
        }
        res.json({status:200, data:modified_results});
    });

    /*
    (SELECT COUNT(*) FROM posts_likes WHERE post_id = posts.id) as posts_likes,
    (SELECT COUNT(*) FROM posts_comments WHERE post_id = posts.id) as comments_count,
    (SELECT COUNT(*) FROM posts_likes WHERE post_id = posts.id AND user_id = ?) as aware
    */
});

router.post('/create', upload.array('files'), async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    var content = req.body.content;
    var files = req.files;

    if (!id || !token || !content) {
        res.json({ status: 400, error: 'Invalid Request', message: 'Please provide the required fields' });
        return;
    }

    var auth = await authjs.checkToken(id, token);
    if (auth == false) {
        res.json({ status: 401, error: 'Invalid Credentials', message: 'Please provide the valid credentials' });
        return;
    }

    if(content.length > 3500){
        res.json({ status: 400, error: 'Invalid Request', message: 'Content is too long' });
        return;
    }

    var post_id = parseInt(new Date().valueOf().toString().substr(0,10));
    var post = {
        id:post_id,
        user_id:id,
        content,
        date:moment().format("YYYY-MM-DD HH:mm:ss")
    };

    var query = "INSERT INTO posts SET ?";
    pool.query(query, post, async (err, results)=>{
        if(err){
            res.json({status:500, error:'Internal Server Error', message:'Something went wrong', err});
        }else{
            var get_post = await postFunctions.getPost(post_id);
            if(get_post == false){
                res.json({status:200, error:'Internal Server Error', message:'Could not retrieve post'});
                return;
            }
            post = new postModels.Post(get_post);
            var ok_uploads = [];
            for (let i = 0; i < files.length; i++) {
                const file = files[i];
                var media_upload = await postFunctions.uploadMedia(post_id, file);
                
                if(media_upload != false){
                    ok_uploads.push(media_upload);
                }
            }
            post.files = ok_uploads;
            res.json({status:200, message:'Ok', post});
        }
    });
});

router.post('/like', upload.none(), async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    var post_id = req.body.post_id;
    if (!id && !token) {
        res.json({ status: 401, error: 'Invalid Credentials', message: 'Please provide the valid credentials' });
        return;
    }

    var authentication = await authjs.checkToken(id, token);
    if (authentication == false) {
        res.json({ status: 401, error: 'Invalid Credentials', message: 'Please provide the valid credentials' });
        return;
    }

    if (!post_id) {
        res.json({ status: 400, error: "Invalid Request" });
        return;
    }

    var postExistence = await postFunctions.checkPostExistence(post_id);
    if (postExistence == false) {
        res.json({ status: 400, error: "Invalid Request" });
        return;
    }

    var awarenessPointExistence = await postFunctions.checkLikeExistence(post_id, id);
    if (awarenessPointExistence == true) {
        res.json({ status: 200, message: 'Ok' });
        return;
    }

    var date = moment().format('YYYY-MM-DD HH:mm:ss');
    var query = 'INSERT INTO posts_likes (post_id, user_id, date) VALUES (?,?,?)';
    pool.query(query, [post_id, id, date], (err, results) => {
        if (err) {
            res.json({ status: 500, error: 'Internal Server Error', message: 'Something went wrong' });
        } else {
            res.json({ status: 200, message: 'Ok' });
        }
    });
});

router.post('/unlike', upload.none(), async (req, res) => {
    var id = req.cookies.c_id;
    var token = req.cookies.c_t;
    var post_id = req.body.post_id;
    if (!id && !token) {
        res.json({ status: 401, error: 'Invalid Credentials', message: 'Please provide the valid credentials' });
        return;
    }

    var authentication = await authjs.checkToken(id, token);
    if (authentication == false) {
        res.json({ status: 401, error: 'Invalid Credentials', message: 'Please provide the valid credentials' });
        return;
    }

    if (!post_id) {
        res.json({ status: 400, error: "Invalid Request" });
        return;
    }

    var postExistence = await postFunctions.checkPostExistence(post_id);
    if (postExistence == false) {
        res.json({ status: 400, error: "Invalid Request" });
        return;
    }

    var awarenessPointExistence = await postFunctions.checkLikeExistence(post_id, id);
    if (awarenessPointExistence == false) {
        res.json({ status: 200, message: 'Ok' });
        return;
    }

    var query = 'DELETE FROM posts_likes WHERE post_id = ? AND user_id = ?';
    pool.query(query, [post_id, id], (err, results) => {
        if (err) {
            res.json({ status: 500, error: 'Internal Server Error', message: 'Something went wrong' });
        } else {
            res.json({ status: 200, message: 'Ok' });
        }
    });
});

module.exports = router;
//import mysql pool
const pool = require('../database');